@@ -5,15 +5,20 @@ module Slim
   class Parser < Temple::Parser
     define_options :file,
                    :default_tag,
-                   :tabsize => 4,
-                   :shortcut => {
-                     '#' => { :attr => 'id' },
-                     '.' => { :attr => 'class' }
-                   },
-                   :attr_delims => {
+                   tabsize: 4,
+                   code_attr_delims: {
                      '(' => ')',
                      '[' => ']',
                      '{' => '}',
+                   },
+                   attr_list_delims: {
+                     '(' => ')',
+                     '[' => ']',
+                     '{' => '}',
+                   },
+                   shortcut: {
+                     '#' => { attr: 'id' },
+                     '.' => { attr: 'class' }
                    }
 
     class SyntaxError < StandardError
@@ -40,6 +45,8 @@ module Slim
 
     def initialize(opts = {})
       super
+      @attr_list_delims = options[:attr_list_delims]
+      @code_attr_delims = options[:code_attr_delims]
       tabsize = options[:tabsize]
       if tabsize > 1
         @tab_re = /\G((?: {#{tabsize}})*) {0,#{tabsize-1}}\t/
@@ -54,16 +61,22 @@ module Slim
         @tag_shortcut[k] = v[:tag] || options[:default_tag]
         if v.include?(:attr)
           @attr_shortcut[k] = [v[:attr]].flatten
-          raise ArgumentError, 'You can only use special characters for attribute shortcuts' if k =~ /(#{WORD_RE}|-)/
+          raise ArgumentError, 'You can only use special characters for attribute shortcuts' if k =~ /(\p{Word}|-)/
         end
       end
       keys = Regexp.union @attr_shortcut.keys.sort_by {|k| -k.size }
-      @attr_shortcut_re = /\A(#{keys}+)(#{WORD_RE}(?:#{WORD_RE}|-)*#{WORD_RE}|#{WORD_RE}+)/
+      @attr_shortcut_re = /\A(#{keys}+)((?:\p{Word}|-)*)/
       keys = Regexp.union @tag_shortcut.keys.sort_by {|k| -k.size }
-      @tag_re = /\A(?:#{keys}|\*(?=[^\s]+)|(#{WORD_RE}(?:#{WORD_RE}|:|-)*#{WORD_RE}|#{WORD_RE}+))/
-      keys = Regexp.escape options[:attr_delims].keys.join
-      @delim_re = /\A[#{keys}]/
-      @attr_delim_re = /\A\s*([#{keys}])/
+      @tag_re = /\A(?:#{keys}|\*(?=[^\s]+)|(\p{Word}(?:\p{Word}|:|-)*\p{Word}|\p{Word}+))/
+      keys = Regexp.escape @code_attr_delims.keys.join
+      @code_attr_delims_re = /\A[#{keys}]/
+      keys = Regexp.escape @attr_list_delims.keys.join
+      @attr_list_delims_re = /\A\s*([#{keys}])/
+      @embedded_re = /\A(#{Regexp.union(Embedded.engines.keys.map(&:to_s))}):(\s*)/
+      keys = Regexp.escape ('"\'></='.split(//) + @attr_list_delims.flatten + @code_attr_delims.flatten).uniq.join
+      @attr_name = "\\A\\s*([^\0\s#{keys}]+)"
+      @quoted_attr_re = /#{@attr_name}\s*=(=?)\s*("|')/
+      @code_attr_re = /#{@attr_name}\s*=(=?)\s*/
     end
 
     # Compile string to Temple expression
@@ -82,11 +95,6 @@ module Slim
 
     protected
 
-    WORD_RE = ''.respond_to?(:encoding) ? '\p{Word}' : '\w'
-    ATTR_NAME = "\\A\\s*(#{WORD_RE}(?:#{WORD_RE}|:|-)*)"
-    QUOTED_ATTR_RE = /#{ATTR_NAME}\s*=(=?)\s*("|')/
-    CODE_ATTR_RE = /#{ATTR_NAME}\s*=(=?)\s*/
-
     def reset(lines = nil, stacks = nil)
       # Since you can indent however you like in Slim, we need to keep a list
       # of how deeply indented you are. For instance, in a template like this:
@@ -100,7 +108,7 @@ module Slim
       #
       # We uses this information to figure out how many steps we must "jump"
       # out when we see an de-indented line.
-      @indents = [0]
+      @indents = []
 
       # Whenever we want to output something, we'll *always* output it to the
       # last stack in this array. So when there's a line that expects
@@ -138,6 +146,9 @@ module Slim
 
       indent = get_indent(@line)
 
+      # Choose first indentation yourself
+      @indents << indent if @indents.empty?
+
       # Remove the indentation
       @line.lstrip!
 
@@ -159,7 +170,7 @@ module Slim
         # This line was deindented.
         # Now we're have to go through the all the indents and figure out
         # how many levels we've deindented.
-        while indent < @indents.last
+        while indent < @indents.last && @indents.size > 1
           @indents.pop
           @stacks.pop
         end
@@ -180,7 +191,7 @@ module Slim
       case @line
       when /\A\/!( ?)/
         # HTML comment
-        @stacks.last << [:html, :comment, [:slim, :text, parse_text_block($', @indents.last + $1.size + 2)]]
+        @stacks.last << [:html, :comment, [:slim, :text, :verbatim, parse_text_block($', @indents.last + $1.size + 2)]]
       when /\A\/\[\s*(.*?)\s*\]\s*\Z/
         # HTML conditional comment
         block = [:multi]
@@ -190,9 +201,9 @@ module Slim
         # Slim comment
         parse_comment_block
       when /\A([\|'])( ?)/
-        # Found a text block.
+        # Found verbatim text block.
         trailing_ws = $1 == "'"
-        @stacks.last << [:slim, :text, parse_text_block($', @indents.last + $2.size + 1)]
+        @stacks.last << [:slim, :text, :verbatim, parse_text_block($', @indents.last + $2.size + 1)]
         @stacks.last << [:static, ' '] if trailing_ws
       when /\A</
         # Inline html
@@ -210,16 +221,20 @@ module Slim
         # Found an output block.
         # We expect the line to be broken or the next line to be indented.
         @line = $'
-        trailing_ws = $2.include?('\'') || $2.include?('>')
+        trailing_ws = $2.include?('>')
+        if $2.include?('\'')
+          deprecated_syntax '=\' for trailing whitespace is deprecated in favor of =>'
+          trailing_ws = true
+        end
         block = [:multi]
         @stacks.last << [:static, ' '] if $2.include?('<')
         @stacks.last << [:slim, :output, $1.empty?, parse_broken_line, block]
         @stacks.last << [:static, ' '] if trailing_ws
         @stacks << block
-      when /\A(\w+):\s*\Z/
+      when @embedded_re
         # Embedded template detected. It is treated as block.
-        @stacks.last << [:slim, :embedded, $1, parse_text_block]
-      when /\Adoctype\s+/i
+        @stacks.last << [:slim, :embedded, $1, parse_text_block($', @orig_line.size - $'.size + $2.size)]
+      when /\Adoctype\b/
         # Found doctype declaration
         @stacks.last << [:html, :doctype, $'.strip]
       when @tag_re
@@ -227,11 +242,18 @@ module Slim
         @line = $' if $1
         parse_tag($&)
       else
-        syntax_error! 'Unknown line indicator'
+        unknown_line_indicator
       end
       @stacks.last << [:newline]
     end
 
+    # Unknown line indicator found. Overwrite this method if
+    # you want to add line indicators to the Slim parser.
+    # The default implementation throws a syntax error.
+    def unknown_line_indicator
+      syntax_error! 'Unknown line indicator'
+    end
+
     def parse_comment_block
       while !@lines.empty? && (@lines.first =~ /\A\s*\Z/ || get_indent(@lines.first) > @indents.last)
         next_line
@@ -239,7 +261,7 @@ module Slim
       end
     end
 
-    def parse_text_block(first_line = nil, text_indent = nil, in_tag = false)
+    def parse_text_block(first_line = nil, text_indent = nil)
       result = [:multi]
       if !first_line || first_line.empty?
         text_indent = nil
@@ -269,11 +291,9 @@ module Slim
           # as deep as the first line.
           offset = text_indent ? indent - text_indent : 0
           if offset < 0
-            syntax_error!("Text line not indented deep enough.\n" +
-                          "The first text line defines the necessary text indentation." +
-                          (in_tag ? "\nAre you trying to nest a child tag in a tag containing text? Use | for the text block!" : ''))
+            text_indent += offset
+            offset = 0
           end
-
           result << [:newline] << [:slim, :interpolate, (text_indent ? "\n" : '') + (' ' * offset) + @line]
 
           # The indentation of first line of the text block
@@ -311,7 +331,12 @@ module Slim
 
       @line =~ /\A[<>']*/
       @line = $'
-      trailing_ws = $&.include?('\'') || $&.include?('>')
+      trailing_ws = $&.include?('>')
+      if $&.include?('\'')
+        deprecated_syntax 'tag\' for trailing whitespace is deprecated in favor of tag>'
+        trailing_ws = true
+      end
+
       leading_ws = $&.include?('<')
 
       parse_attributes(attributes)
@@ -326,20 +351,28 @@ module Slim
       when /\A\s*:\s*/
         # Block expansion
         @line = $'
-        (@line =~ @tag_re) || syntax_error!('Expected tag')
-        @line = $' if $1
-        content = [:multi]
-        tag << content
-        i = @stacks.size
-        @stacks << content
-        parse_tag($&)
-        @stacks.delete_at(i)
+        if @line =~ @embedded_re
+          tag << [:slim, :embedded, $1, parse_text_block($', @orig_line.size - @line.size + $2.size)]
+        else
+          (@line =~ @tag_re) || syntax_error!('Expected tag')
+          @line = $' if $1
+          content = [:multi]
+          tag << content
+          i = @stacks.size
+          @stacks << content
+          parse_tag($&)
+          @stacks.delete_at(i)
+        end
       when /\A\s*=(=?)(['<>]*)/
         # Handle output code
         @line = $'
-        trailing_ws2 = $2.include?('\'') || $2.include?('>')
+        trailing_ws2 = $2.include?('>')
+        if $2.include?('\'')
+          deprecated_syntax '=\' for trailing whitespace is deprecated in favor of =>'
+          trailing_ws2 = true
+        end
         block = [:multi]
-        @stacks.last << [:static, ' '] if !leading_ws && $2.include?('<')
+        @stacks.last.insert(-2, [:static, ' ']) if !leading_ws && $2.include?('<')
         tag << [:slim, :output, $1 != '=', parse_broken_line, block]
         @stacks.last << [:static, ' '] if !trailing_ws && trailing_ws2
         @stacks << block
@@ -352,22 +385,22 @@ module Slim
         content = [:multi]
         tag << content
         @stacks << content
-      when /\A( ?)(.*)\Z/
+      when /\A ?/
         # Text content
-        tag << [:slim, :text, parse_text_block($2, @orig_line.size - @line.size + $1.size, true)]
+        tag << [:slim, :text, :inline, parse_text_block($', @orig_line.size - $'.size)]
       end
     end
 
     def parse_attributes(attributes)
       # Check to see if there is a delimiter right after the tag name
       delimiter = nil
-      if @line =~ @attr_delim_re
-        delimiter = options[:attr_delims][$1]
+      if @line =~ @attr_list_delims_re
+        delimiter = @attr_list_delims[$1]
         @line = $'
       end
 
       if delimiter
-        boolean_attr_re = /#{ATTR_NAME}(?=(\s|#{Regexp.escape delimiter}|\Z))/
+        boolean_attr_re = /#{@attr_name}(?=(\s|#{Regexp.escape delimiter}|\Z))/
         end_re = /\A\s*#{Regexp.escape delimiter}/
       end
 
@@ -377,12 +410,12 @@ module Slim
           # Splat attribute
           @line = $'
           attributes << [:slim, :splat, parse_ruby_code(delimiter)]
-        when QUOTED_ATTR_RE
+        when @quoted_attr_re
           # Value is quoted (static)
           @line = $'
           attributes << [:html, :attr, $1,
                          [:escape, $2.empty?, [:slim, :interpolate, parse_quoted_attribute($3)]]]
-        when CODE_ATTR_RE
+        when @code_attr_re
           # Value is ruby code
           @line = $'
           name = $1
@@ -397,7 +430,7 @@ module Slim
           when boolean_attr_re
             # Boolean attribute
             @line = $'
-            attributes << [:html, :attr, $1, [:slim, :attrvalue, false, 'true']]
+            attributes << [:html, :attr, $1, [:multi]]
           when end_re
             # Find ending delimiter
             @line = $'
@@ -433,9 +466,9 @@ module Slim
             elsif @line[0] == close_delimiter[0]
               count -= 1
             end
-          elsif @line =~ @delim_re
+          elsif @line =~ @code_attr_delims_re
             count = 1
-            delimiter, close_delimiter = $&, options[:attr_delims][$&]
+            delimiter, close_delimiter = $&, @code_attr_delims[$&]
           end
           code << @line.slice!(0)
         end
@@ -447,29 +480,21 @@ module Slim
     def parse_quoted_attribute(quote)
       value, count = '', 0
 
-      until @line.empty? || (count == 0 && @line[0] == quote[0])
-        if @line =~ /\A\\\Z/
-          value << ' '
+      until count == 0 && @line[0] == quote[0]
+        if @line =~ /\A(\\)?\Z/
+          value << ($1 ? ' ' : "\n")
           expect_next_line
         else
-          if count > 0
-            if @line[0] == ?{
-              count += 1
-            elsif @line[0] == ?}
-              count -= 1
-            end
-          elsif @line =~ /\A#\{/
-            value << @line.slice!(0)
-            count = 1
+          if @line[0] == ?{
+            count += 1
+          elsif @line[0] == ?}
+            count -= 1
           end
           value << @line.slice!(0)
         end
       end
 
-      syntax_error!("Expected closing brace }") if count != 0
-      syntax_error!("Expected closing quote #{quote}") if @line[0] != quote[0]
       @line.slice!(0)
-
       value
     end
 
@@ -477,6 +502,21 @@ module Slim
     def syntax_error!(message)
       raise SyntaxError.new(message, options[:file], @orig_line, @lineno,
                             @orig_line && @line ? @orig_line.size - @line.size : 0)
+    rescue SyntaxError => ex
+      # HACK: Manipulate stacktrace for Rails and other frameworks
+      # to find the right file.
+      ex.backtrace.unshift "#{options[:file]}:#{@lineno}"
+      raise
+    end
+
+    def deprecated_syntax(message)
+      line = @orig_line.lstrip
+      column = (@orig_line && @line ? @orig_line.size - @line.size : 0) + line.size - @orig_line.size
+      warn %{Deprecated syntax: #{message}
+  #{options[:file]}, Line #{@lineno}, Column #{@column}
+    #{line}
+    #{' ' * column}^
+}
     end
 
     def expect_next_line
