@@ -1,13 +1,19 @@
 # encoding: utf-8
 
+begin
+  require 'codeclimate-test-reporter'
+  CodeClimate::TestReporter.start
+rescue LoadError
+end
+
 require 'minitest/autorun'
 require 'slim'
 require 'slim/grammar'
 
-Slim::Engine.after  Slim::Parser, Temple::Filters::Validator, :grammar => Slim::Grammar
+Slim::Engine.after  Slim::Parser, Temple::Filters::Validator, grammar: Slim::Grammar
 Slim::Engine.before :Pretty, Temple::Filters::Validator
 
-class TestSlim < MiniTest::Unit::TestCase
+class TestSlim < Minitest::Test
   def setup
     @env = Env.new
   end
@@ -46,6 +52,8 @@ class TestSlim < MiniTest::Unit::TestCase
     raise 'Syntax error expected'
   rescue Slim::Parser::SyntaxError => ex
     assert_equal message, ex.message
+    message =~ /([^\s]+), Line (\d+)/
+    assert_backtrace ex, "#{$1}:#{$2}"
   end
 
   def assert_ruby_error(error, from, source, options = {})
@@ -58,12 +66,12 @@ class TestSlim < MiniTest::Unit::TestCase
   def assert_backtrace(ex, from)
     if defined?(RUBY_ENGINE) && RUBY_ENGINE == 'rbx'
       # HACK: Rubinius stack trace sometimes has one entry more
-      if ex.backtrace[0] !~ /^#{Regexp.escape from}:/
-        ex.backtrace[1] =~ /([^\s]+:\d+):/
+      if ex.backtrace[0] !~ /^#{Regexp.escape from}/
+        ex.backtrace[1] =~ /([^\s]+:\d+)/
         assert_equal from, $1
       end
     else
-      ex.backtrace[0] =~ /([^\s]+:\d+):/
+      ex.backtrace[0] =~ /([^\s]+:\d+)/
       assert_equal from, $1
     end
   end
@@ -97,7 +105,7 @@ class Env
   end
 
   def hash
-    {:a => 'The letter a', :b => 'The letter b'}
+    {a: 'The letter a', b: 'The letter b'}
   end
 
   def show_first?(show = false)
@@ -155,7 +163,7 @@ class ViewEnv
   end
 
   def person
-    [{:name => 'Joe'}, {:name => 'Jack'}]
+    [{name: 'Joe'}, {name: 'Jack'}]
   end
 
   def people
