@@ -1,4 +1,5 @@
 require 'helper'
+require 'erb' #asciidoctor fail to load it randomly
 
 class TestSlimEmbeddedEngines < TestSlim
   def test_render_with_erb
@@ -51,7 +52,7 @@ markdown:
       # kramdown, :auto_ids by default
       assert_html "<h1 id=\"header\">Header</h1>\n<p>Hello from Markdown!</p>\n\n<p>3</p>\n\n<ul>\n  <li>one</li>\n  <li>two</li>\n</ul>\n", source
 
-      Slim::Embedded.with_options(:markdown => {:auto_ids => false}) do
+      Slim::Embedded.with_options(markdown: {auto_ids: false}) do
         assert_html "<h1>Header</h1>\n<p>Hello from Markdown!</p>\n\n<p>3</p>\n\n<ul>\n  <li>one</li>\n  <li>two</li>\n</ul>\n", source
       end
 
@@ -68,6 +69,17 @@ creole:
     assert_html "<h1>head1</h1><h2>head2</h2>", source
   end
 
+  def test_render_with_creole_one_line
+    source = %q{
+creole: Hello **world**,
+  we can write one-line embedded markup now!
+  = Headline
+  Text
+.nested: creole: **Strong**
+}
+    assert_html '<p>Hello <strong>world</strong>, we can write one-line embedded markup now!</p><h1>Headline</h1><p>Text</p><div class="nested"><p><strong>Strong</strong></p></div>', source
+  end
+
   def test_render_with_org
     # HACK: org-ruby registers itself in Tilt
     require 'org-ruby'
@@ -83,7 +95,7 @@ org:
   def test_render_with_builder
     source = %q{
 builder:
-  xml.p(:id => 'test') {
+  xml.p(id: 'test') {
     xml.text!('Hello')
   }
 }
@@ -109,12 +121,27 @@ javascript:
   alert('hello')
 p Hi
 }
-    assert_html %{<script type="text/javascript">$(function() {});\n\n\nalert('hello')</script><p>Hi</p>}, source
+    assert_html %{<script>$(function() {});\n\n\nalert('hello')</script><p>Hi</p>}, source
+  end
+
+  def test_render_with_opal
+    begin
+      # HACK: org-ruby registers itself in Tilt
+      require 'opal'
+    rescue LoadError
+      return
+    end
+
+    source = %q{
+opal:
+  puts 'hello from opal'
+}
+    assert_match '$puts("hello from opal")', render(source)
   end
 
   def test_render_with_javascript_with_tabs
     source = "javascript:\n\t$(function() {});\n\talert('hello')\np Hi"
-    assert_html "<script type=\"text/javascript\">$(function() {});\nalert('hello')</script><p>Hi</p>", source
+    assert_html "<script>$(function() {});\nalert('hello')</script><p>Hi</p>", source
   end
 
   def test_render_with_javascript_including_variable
@@ -124,34 +151,34 @@ p Hi
 javascript:   
   $(function() { #{func} });
 }
-    assert_html %q|<script type="text/javascript">$(function() { alert(&#39;hello&#39;); });</script>|, source
+    assert_html %q|<script>$(function() { alert(&#39;hello&#39;); });</script>|, source
   end
 
   def test_render_with_javascript_with_explicit_html_comment
-    Slim::Engine.with_options(:js_wrapper => :comment) do
+    Slim::Engine.with_options(js_wrapper: :comment) do
       source = "javascript:\n\t$(function() {});\n\talert('hello')\np Hi"
-      assert_html "<script type=\"text/javascript\"><!--\n$(function() {});\nalert('hello')\n//--></script><p>Hi</p>", source
+      assert_html "<script><!--\n$(function() {});\nalert('hello')\n//--></script><p>Hi</p>", source
     end
   end
 
   def test_render_with_javascript_with_explicit_cdata_comment
-    Slim::Engine.with_options(:js_wrapper => :cdata) do
+    Slim::Engine.with_options(js_wrapper: :cdata) do
       source = "javascript:\n\t$(function() {});\n\talert('hello')\np Hi"
-      assert_html "<script type=\"text/javascript\">\n//<![CDATA[\n$(function() {});\nalert('hello')\n//]]>\n</script><p>Hi</p>", source
+      assert_html "<script>\n//<![CDATA[\n$(function() {});\nalert('hello')\n//]]>\n</script><p>Hi</p>", source
     end
   end
 
   def test_render_with_javascript_with_format_xhtml_comment
-    Slim::Engine.with_options(:js_wrapper => :guess, :format => :xhtml) do
+    Slim::Engine.with_options(js_wrapper: :guess, format: :xhtml) do
       source = "javascript:\n\t$(function() {});\n\talert('hello')\np Hi"
-      assert_html "<script type=\"text/javascript\">\n//<![CDATA[\n$(function() {});\nalert('hello')\n//]]>\n</script><p>Hi</p>", source
+      assert_html "<script>\n//<![CDATA[\n$(function() {});\nalert('hello')\n//]]>\n</script><p>Hi</p>", source
     end
   end
 
   def test_render_with_javascript_with_format_html_comment
-    Slim::Engine.with_options(:js_wrapper => :guess, :format => :html) do
+    Slim::Engine.with_options(js_wrapper: :guess, format: :html) do
       source = "javascript:\n\t$(function() {});\n\talert('hello')\np Hi"
-      assert_html "<script type=\"text/javascript\"><!--\n$(function() {});\nalert('hello')\n//--></script><p>Hi</p>", source
+      assert_html "<script><!--\n$(function() {});\nalert('hello')\n//--></script><p>Hi</p>", source
     end
   end
 
@@ -179,22 +206,22 @@ scss:
 ruby:
   Embedded Ruby
 }
-    assert_runtime_error 'Embedded engine ruby is disabled', source, :enable_engines => [:javascript]
-    assert_runtime_error 'Embedded engine ruby is disabled', source, :enable_engines => %w(javascript)
+    assert_runtime_error 'Embedded engine ruby is disabled', source, enable_engines: [:javascript]
+    assert_runtime_error 'Embedded engine ruby is disabled', source, enable_engines: %w(javascript)
 
     source = %{
 ruby:
   Embedded Ruby
 }
-    assert_runtime_error 'Embedded engine ruby is disabled', source, :enable_engines => [:javascript]
-    assert_runtime_error 'Embedded engine ruby is disabled', source, :enable_engines => %w(javascript)
+    assert_runtime_error 'Embedded engine ruby is disabled', source, enable_engines: [:javascript]
+    assert_runtime_error 'Embedded engine ruby is disabled', source, enable_engines: %w(javascript)
 
     source = %{
 ruby:
   Embedded Ruby
 }
-    assert_runtime_error 'Embedded engine ruby is disabled', source, :disable_engines => [:ruby]
-    assert_runtime_error 'Embedded engine ruby is disabled', source, :disable_engines => %w(ruby)
+    assert_runtime_error 'Embedded engine ruby is disabled', source, disable_engines: [:ruby]
+    assert_runtime_error 'Embedded engine ruby is disabled', source, disable_engines: %w(ruby)
   end
 
   def test_enabled_embedded_engine
@@ -202,14 +229,14 @@ ruby:
 javascript:
   $(function() {});
 }
-    assert_html '<script type="text/javascript">$(function() {});</script>', source, :disable_engines => [:ruby]
-    assert_html '<script type="text/javascript">$(function() {});</script>', source, :disable_engines => %w(ruby)
+    assert_html '<script>$(function() {});</script>', source, disable_engines: [:ruby]
+    assert_html '<script>$(function() {});</script>', source, disable_engines: %w(ruby)
 
     source = %q{
 javascript:
   $(function() {});
 }
-    assert_html '<script type="text/javascript">$(function() {});</script>', source, :enable_engines => [:javascript]
-    assert_html '<script type="text/javascript">$(function() {});</script>', source, :enable_engines => %w(javascript)
+    assert_html '<script>$(function() {});</script>', source, enable_engines: [:javascript]
+    assert_html '<script>$(function() {});</script>', source, enable_engines: %w(javascript)
   end
 end
