@@ -14,6 +14,15 @@ html
     assert_html '<html><head><title>Simple Test Title</title></head><body><p>Hello World, meet Slim.</p></body></html>', source
   end
 
+  def test_relaxed_indentation_of_first_line
+    source = %q{
+  p
+    .content
+}
+
+    assert_html "<p><div class=\"content\"></div></p>", source
+  end
+
   def test_html_tag_with_text_and_empty_line
     source = %q{
 p Hello
@@ -39,7 +48,7 @@ doctype 1.1
 html
 }
 
-    assert_html '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd"><html></html>', source, :format => :xhtml
+    assert_html '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd"><html></html>', source, format: :xhtml
   end
 
   def test_doctype_new_syntax
@@ -48,7 +57,7 @@ doctype 5
 html
 }
 
-    assert_html '<!DOCTYPE html><html></html>', source, :format => :xhtml
+    assert_html '<!DOCTYPE html><html></html>', source, format: :xhtml
   end
 
   def test_doctype_new_syntax_html5
@@ -57,7 +66,7 @@ doctype html
 html
 }
 
-    assert_html '<!DOCTYPE html><html></html>', source, :format => :xhtml
+    assert_html '<!DOCTYPE html><html></html>', source, format: :xhtml
   end
 
   def test_render_with_shortcut_attributes
@@ -76,7 +85,7 @@ h1#title This is my title
    = hello_world
  }
 
-    assert_html '<section class="hello world" id="notice">Hello World from @env</section>', source, :default_tag => 'section'
+    assert_html '<section class="hello world" id="notice">Hello World from @env</section>', source, default_tag: 'section'
   end
 
   def test_render_with_custom_shortcut
@@ -87,7 +96,7 @@ h1#title This is my title
   = hello_world
 }
 
-    assert_html '<div class="hello world" id="notice" role="test">Hello World from @env</div><section role="abc">Hello World from @env</section>', source, :shortcut => {'#' => {:attr => 'id'}, '.' => {:attr => 'class'}, '@' => {:tag => 'section', :attr => 'role'}}
+    assert_html '<div class="hello world" id="notice" role="test">Hello World from @env</div><section role="abc">Hello World from @env</section>', source, shortcut: {'#' => {attr: 'id'}, '.' => {attr: 'class'}, '@' => {tag: 'section', attr: 'role'}}
   end
 
   def test_render_with_text_block
@@ -209,6 +218,17 @@ p#test class="paragraph" This is line one.
     assert_html "<p class=\"paragraph\" id=\"test\">This is line one.\nThis is line two.</p>", source
   end
 
+  def test_relaxed_text_indentation
+    source = %q{
+p
+  | text block
+   text
+    line3
+}
+
+    assert_html "<p>text block\ntext\n line3</p>", source
+  end
+
   def test_output_code_with_leading_spaces
     source = %q{
 p= hello_world
@@ -287,7 +307,7 @@ p(id="marvin" class="martian" data-info="Illudium Q-36")= output_number
     source = %q{
 p<id="marvin" class="martian" data-info="Illudium Q-36">= output_number
 }
-    Slim::Parser.default_options[:attr_delims].each do |k,v|
+    Slim::Parser.options[:attr_list_delims].each do |k,v|
       str = source.sub('<',k).sub('>',v)
       assert_html '<p class="martian" data-info="Illudium Q-36" id="marvin">1337</p>', str
     end
@@ -299,8 +319,8 @@ p { foo="bar" }
 }
 
     assert_html '<p foo="bar"></p>', source
-    assert_html '<p foo="bar"></p>', source, :attr_delims => {'{' => '}'}
-    assert_html '<p>{ foo="bar" }</p>', source, :attr_delims => {'(' => ')', '[' => ']'}
+    assert_html '<p foo="bar"></p>', source, attr_list_delims: {'{' => '}'}
+    assert_html '<p>{ foo="bar" }</p>', source, attr_list_delims: {'(' => ')', '[' => ']'}
   end
 
   def test_closed_tag
@@ -308,7 +328,26 @@ p { foo="bar" }
 closed/
 }
 
-    assert_html '<closed />', source, :format => :xhtml
+    assert_html '<closed />', source, format: :xhtml
+  end
+
+  def test_custom_attr_list_delims_option
+    source = %q{
+p { foo="bar" x=(1+1) }
+p < x=(1+1) > Hello
+}
+
+    assert_html '<p foo="bar" x="2"></p><p>< x=(1+1) > Hello</p>', source
+    assert_html '<p foo="bar" x="2"></p><p>< x=(1+1) > Hello</p>', source, attr_list_delims: {'{' => '}'}
+    assert_html '<p>{ foo="bar" x=(1+1) }</p><p x="2">Hello</p>', source, attr_list_delims: {'<' => '>'}, code_attr_delims: { '(' => ')' }
+  end
+
+  def test_closed_tag
+    source = %q{
+closed/
+}
+
+    assert_html '<closed />', source, format: :xhtml
   end
 
   def test_attributs_with_parens_and_spaces
@@ -331,7 +370,7 @@ closed/
 closed id="test" /
 }
 
-    assert_html '<closed id="test" />', source, :format => :xhtml
+    assert_html '<closed id="test" />', source, format: :xhtml
   end
 
   def test_closed_tag_with_attributes_and_parens
@@ -339,7 +378,7 @@ closed id="test" /
 closed(id="test")/
 }
 
-    assert_html '<closed id="test" />', source, :format => :xhtml
+    assert_html '<closed id="test" />', source, format: :xhtml
   end
 
   def test_render_with_html_comments
@@ -378,7 +417,7 @@ p<id="marvin"
 class="martian"
  data-info="Illudium Q-36"> = output_number
 }
-    Slim::Parser.default_options[:attr_delims].each do |k,v|
+    Slim::Parser.options[:attr_list_delims].each do |k,v|
       str = source.sub('<',k).sub('>',v)
       assert_html '<p class="martian" data-info="Illudium Q-36" id="marvin">1337</p>', str
     end
@@ -390,7 +429,7 @@ p<id="marvin"
   class="martian"
  data-info="Illudium Q-36"> THE space modulator
 }
-    Slim::Parser.default_options[:attr_delims].each do |k,v|
+    Slim::Parser.options[:attr_list_delims].each do |k,v|
       str = source.sub('<',k).sub('>',v)
       assert_html '<p class="martian" data-info="Illudium Q-36" id="marvin">THE space modulator</p>', str
     end
@@ -403,7 +442,7 @@ p<id="marvin"
 data-info="Illudium Q-36">
   | THE space modulator
 }
-    Slim::Parser.default_options[:attr_delims].each do |k,v|
+    Slim::Parser.options[:attr_list_delims].each do |k,v|
       str = source.sub('<',k).sub('>',v)
       assert_html '<p class="martian" data-info="Illudium Q-36" id="marvin">THE space modulator</p>', str
     end
@@ -416,7 +455,7 @@ p<id=id_helper
   data-info="Illudium Q-36">
   | THE space modulator
 }
-    Slim::Parser.default_options[:attr_delims].each do |k,v|
+    Slim::Parser.options[:attr_list_delims].each do |k,v|
       str = source.sub('<',k).sub('>',v)
       assert_html '<p class="martian" data-info="Illudium Q-36" id="notice">THE space modulator</p>', str
     end
@@ -430,7 +469,7 @@ p<id=id_helper
   span.emphasis THE
   |  space modulator
 }
-    Slim::Parser.default_options[:attr_delims].each do |k,v|
+    Slim::Parser.options[:attr_list_delims].each do |k,v|
       str = source.sub('<',k).sub('>',v)
       assert_html '<p class="martian" data-info="Illudium Q-36" id="notice"><span class="emphasis">THE</span> space modulator</p>', str
     end
@@ -443,7 +482,7 @@ li< id="myid"
 data-info="myinfo">
   a href="link" My Link
 }
-    Slim::Parser.default_options[:attr_delims].each do |k,v|
+    Slim::Parser.options[:attr_list_delims].each do |k,v|
       str = source.sub('<',k).sub('>',v)
       assert_html '<li class="myclass" data-info="myinfo" id="myid"><a href="link">My Link</a></li>', str
     end
